/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::SkillsMemory
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/interface/observers/ObserverInterface.h>

#include <ArmarXGui/libraries/ArmarXGuiComponentPlugins/LightweightRemoteGuiComponentPlugin.h>

#include <RobotAPI/libraries/RobotAPIComponentPlugins/ArVizComponentPlugin.h>
#include <RobotAPI/libraries/armem/server/ComponentPlugin.h>
#include <RobotAPI/libraries/armem_skills/aron/Statechart.aron.generated.h>
#include <RobotAPI/libraries/armem_skills/StatechartListener.h>


namespace armarx
{
    /**
     * @defgroup Component-SkillsMemory SkillsMemory
     * @ingroup RobotAPI-Components
     * A description of the component SkillsMemory.
     *
     * @class SkillsMemory
     * @ingroup Component-SkillsMemory
     * @brief Brief description of class SkillsMemory.
     *
     * Detailed description of class SkillsMemory.
     */
    class SkillsMemory :
        virtual public armarx::Component,
        virtual public armem::server::ComponentPluginUser
    // , virtual public armarx::ArVizComponentPluginUser
    {
    public:
        SkillsMemory();

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;

        // WritingInterface interface
    public:
        armem::data::AddSegmentsResult addSegments(const armem::data::AddSegmentsInput& input, const Ice::Current&) override;
        armem::data::CommitResult commit(const armem::data::Commit& commit, const Ice::Current&) override;


    protected:
        /// @see armarx::ManagedIceObject::onInitComponent()
        void onInitComponent() override;

        /// @see armarx::ManagedIceObject::onConnectComponent()
        void onConnectComponent() override;

        /// @see armarx::ManagedIceObject::onDisconnectComponent()
        void onDisconnectComponent() override;

        /// @see armarx::ManagedIceObject::onExitComponent()
        void onExitComponent() override;

        /// @see PropertyUser::createPropertyDefinitions()
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;


    private:
        DebugObserverInterfacePrx debugObserver;

        struct Properties
        {
            std::string memoryName = "Skills";

            std::string statechartTransitionsTopicName = "StateReportingTopic";

            struct CoreSegments
            {
                bool addOnUsage = false;
            };
            CoreSegments core;
        };
        Properties p;

        // Statechart transition logging
        const std::string statechartCoreSegmentName = "Statechart";
        const std::string statechartTransitionsProviderSegmentName = "Transitions";

        IceInternal::Handle<StatechartListener> createStatechartListener(const std::string& topicName,
                const std::string& _name = "");
        IceInternal::Handle<armarx::StatechartListener> statechartListener;
        void reportTransitions(const std::vector<StatechartListener::Transition>& transitions);

        // Gets the statechart name from a state name (takes first two levels of the hierarchy)
        static std::string getStatechartName(std::string stateName);
    };
}
