#include "MemoryConnector.h"

#include <ArmarXCore/core/ManagedIceObject.h>

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

namespace armarx::armem
{

    MemoryConnector::MemoryConnector(ManagedIceObject& component) : component(component) {}

    MemoryConnector::~MemoryConnector() = default;

    void MemoryConnector::registerPropertyDefinitions(PropertyDefinitionsPtr& def)
    {
        const std::string prefix = getPropertyPrefix();

        ARMARX_INFO << "Memory connector: registerPropertyDefinitions with prefix " << prefix;


        def->component(memoryNameSystem, "ArMemMemoryNameSystem", prefix + "ArMemMemoryNameSystem");
    }

    armem::data::WaitForMemoryResult MemoryConnector::useMemory(const std::string& memoryName)
    {
        armem::data::WaitForMemoryInput input;
        input.name = memoryName;

        ARMARX_CHECK_NOT_NULL(memoryNameSystem);

        ARMARX_INFO << "Waiting for memory ...";
        armem::data::WaitForMemoryResult result = memoryNameSystem->waitForMemory(input);

        if (result.success)
        {
            // Add dependency.
            component.usingProxy(result.proxy->ice_getIdentity().name);
        }
        else
        {
            ARMARX_WARNING << "Use memory: Failure";
        }
        return result;
    }

    const std::string& MemoryConnector::getPropertyPrefix() const
    {
        return propertyPrefix;
    }


} // namespace armarx::armem
