#include "EntityInstance.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


std::ostream& armarx::armem::wm::operator<<(std::ostream& os, const EntityInstanceMetadata& d)
{
    os << "EntityInstanceMetadata: "
       << "\n- t_create =   \t" << armem::toStringMicroSeconds(d.timeCreated) << " us"
       << "\n- t_sent =     \t" << armem::toStringMicroSeconds(d.timeSent) << " us"
       << "\n- t_arrived =  \t" << armem::toStringMicroSeconds(d.timeArrived) << " us"
       << "\n- confidence = \t" << d.confidence << " us"
          ;
    return os;
}

namespace armarx::armem::wm
{

    bool EntityInstanceMetadata::operator==(const EntityInstanceMetadata& other) const
    {
        return timeCreated == other.timeCreated
               && timeSent == other.timeSent
               && timeArrived == other.timeArrived
               && std::abs(confidence - other.confidence) < 1e-6f;
    }


    bool EntityInstance::equalsDeep(const EntityInstance& other) const
    {
        return id() == other.id()
               && _metadata == other.metadata()
               && _data->equalsDeep(other.data());
    }

    void EntityInstance::update(const EntityUpdate& update, int index)
    {
        ARMARX_CHECK_FITS_SIZE(index, update.instancesData.size());

        this->index() = index;
        setData(update.instancesData.at(size_t(index)));

        this->_metadata.confidence = update.confidence;

        this->_metadata.timeCreated = update.timeCreated;
        this->_metadata.timeSent = update.timeSent;
        this->_metadata.timeArrived = update.timeArrived;
    }

    EntityInstance EntityInstance::copy() const
    {
        EntityInstance d;
        this->_copySelf(d);
        return d;
    }

    void EntityInstance::_copySelf(EntityInstance& other) const
    {
        Base::_copySelf(other);
        other._metadata = _metadata;
        other._data = _data;
    }

    void EntityInstance::_copySelfWithoutData(EntityInstance& other) const
    {
        Base::_copySelf(other);
        other._metadata = _metadata;
    }
}
