#include "aron_conversions.h"

#include <RobotAPI/libraries/ArmarXObjects/aron_conversions.h>

#include <RobotAPI/libraries/aron/common/aron_conversions/core.h>
#include <RobotAPI/libraries/aron/common/aron_conversions/stl.h>
#include <RobotAPI/libraries/aron/common/aron_conversions/armarx.h>

namespace armarx::armem
{

    void fromAron(const arondto::ObjectInstance& dto,
                  objpose::arondto::ObjectPose& bo)
    {
        bo = dto.pose;
    }

    void toAron(arondto::ObjectInstance& dto, const objpose::arondto::ObjectPose& bo)
    {
        dto.pose = bo;
    }

    void fromAron(const arondto::ObjectInstance& dto, objpose::ObjectPose& bo)
    {
        objpose::fromAron(dto.pose, bo);
    }

    void toAron(arondto::ObjectInstance& dto, const objpose::ObjectPose& bo)
    {
        objpose::toAron(dto.pose, bo);
    }


armarx::armem::MemoryID
armarx::armem::obj::makeObjectInstanceMemoryID(const objpose::ObjectPose& objectPose)
{
    return MemoryID("Object/Instance")
           .withProviderSegmentName(objectPose.providerName)
           .withEntityName(objectPose.objectID.str())
           .withTimestamp(objectPose.timestamp);
}
    /* to be moved */
    void fromAron(const long& dto, IceUtil::Time& time)
    {
        time = IceUtil::Time::microSeconds(dto);
    }

    void toAron(long& dto, const IceUtil::Time& time)
    {
        dto = time.toMicroSeconds();
    }


    /* Robot */

    void fromAron(const arondto::Robot& dto, Robot& bo)
    {
        // fromAron(dto.description, bo.description);
        fromAron(dto.state, bo.config);
    }

    void toAron(arondto::Robot& dto, const Robot& bo)
    {
        // toAron(dto.description, bo.description);
        toAron(dto.state, bo.config);
    }

    /* RobotDescription */

    void fromAron(const arondto::RobotDescription& dto, RobotDescription& bo)
    {
        aron::fromAron(dto.name, bo.name);
        fromAron(dto.xml, bo.xml);
    }

    void toAron(arondto::RobotDescription& dto, const RobotDescription& bo)
    {
        aron::toAron(dto.name, bo.name);
        toAron(dto.xml, bo.xml);
    }

    /* RobotState */

    void fromAron(const arondto::RobotState& dto, RobotState& bo)
    {
        fromAron(dto.timestamp, bo.timestamp);
        bo.globalPose.matrix() = dto.globalPose;
        bo.jointMap            = dto.jointMap;
    }

    void toAron(arondto::RobotState& dto, const RobotState& bo)
    {
        toAron(dto.timestamp, bo.timestamp);
        dto.globalPose = bo.globalPose.matrix();
        dto.jointMap   = bo.jointMap;
    }

}  // namespace armarx::armem
