/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "SerializerFactory.h"

// ArmarX
#include <RobotAPI/libraries/aron/core/codegenerator/codeWriter/cpp/serializer/AllSerializers.h>

namespace armarx::aron::cppcodegenerator
{
    // Access method
    SerializerPtr SerializerFactory::create(const typenavigator::NavigatorPtr& n, const Path& path) const
    {
        static const std::map<type::Descriptor, SerializerFactoryPtr> Factories =
        {
            {type::Descriptor::eObject, SerializerFactoryPtr(new ObjectSerializerFactory())},
            {type::Descriptor::eDict, SerializerFactoryPtr(new DictSerializerFactory())},
            {type::Descriptor::eList, SerializerFactoryPtr(new ListSerializerFactory())},
            {type::Descriptor::eTuple, SerializerFactoryPtr(new TupleSerializerFactory())},
            {type::Descriptor::ePair, SerializerFactoryPtr(new PairSerializerFactory())},
            {type::Descriptor::eEigenMatrix, SerializerFactoryPtr(new EigenMatrixSerializerFactory())},
            {type::Descriptor::eEigenQuaternion, SerializerFactoryPtr(new EigenQuaternionSerializerFactory())},
            {type::Descriptor::eIVTCByteImage, SerializerFactoryPtr(new IVTCByteImageSerializerFactory())},
            {type::Descriptor::eOpenCVMat, SerializerFactoryPtr(new OpenCVMatSerializerFactory())},
            {type::Descriptor::ePCLPointCloud, SerializerFactoryPtr(new PCLPointCloudSerializerFactory())},
            {type::Descriptor::ePosition, SerializerFactoryPtr(new PositionSerializerFactory())},
            {type::Descriptor::eOrientation, SerializerFactoryPtr(new OrientationSerializerFactory())},
            {type::Descriptor::ePose, SerializerFactoryPtr(new PoseSerializerFactory())},
            {type::Descriptor::eIntEnum, SerializerFactoryPtr(new IntEnumSerializerFactory())},
            {type::Descriptor::eInt, SerializerFactoryPtr(new IntSerializerFactory())},
            {type::Descriptor::eLong, SerializerFactoryPtr(new LongSerializerFactory())},
            {type::Descriptor::eFloat, SerializerFactoryPtr(new FloatSerializerFactory())},
            {type::Descriptor::eDouble, SerializerFactoryPtr(new DoubleSerializerFactory())},
            {type::Descriptor::eString, SerializerFactoryPtr(new StringSerializerFactory())},
            {type::Descriptor::eBool, SerializerFactoryPtr(new BoolSerializerFactory())},
            {type::Descriptor::eTime, SerializerFactoryPtr(new TimeSerializerFactory())}
        };

        //CheckIfPtrIsNull("NavigatorFactory", "create", path, n);
        auto res = n->getResult();
        auto factory_iterator = Factories.find(Resolver::GetDescriptor(res));
        if (factory_iterator == Factories.end())
        {
            throw error::AronException("NavigatorFactory", "create", "Cannot find the desired factory for input (ice_id): " + res->ice_id() + ". Cannot create navigator", path);
        }
        return factory_iterator->second->createSpecific(n, path);
    }

    SerializerPtr SerializerFactory::createSpecific(const typenavigator::NavigatorPtr&, const Path&) const
    {
        throw error::AronException("NavigatorFactory", "createSpecific", "Called disallowed method. Use child class instead!");
    }

    // Factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    SerializerPtr upperType##SerializerFactory::createSpecific(const typenavigator::NavigatorPtr& n, const Path& path) const \
    { \
        /*CheckIfInputIsNull("Aron"+std::string(#upperType)+"TypeCppSerializerFactory", "createSpecific[BeforeCast]", path, n); */\
        typenavigator::upperType##NavigatorPtr casted = typenavigator::upperType##Navigator::DynamicCast(n); \
        /*CheckIfInputIsNull("Aron"+std::string(#upperType)+"TypeCppSerializerFactory", "createSpecific[AfterCast]", path, casted); */\
        return SerializerPtr(new serializer::upperType##Serializer(casted)); \
    }

    HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
}
