/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/core/io/dataIO/Writer.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/Concepts.h>
#include <RobotAPI/libraries/aron/core/navigator/data/Navigator.h>
#include <RobotAPI/libraries/aron/core/io/dataIO/writer/navigator/NavigatorWriterToken.h>

namespace armarx::aron::dataIO::writer
{
    class NavigatorWriter;
    typedef std::shared_ptr<NavigatorWriter> NavigatorWriterPtr;

    class NavigatorWriter :
        virtual public WriterInterface
    {
    public:
        NavigatorWriter() = default;

        virtual void writeStartDict() override;
        virtual void writeEndDict() override;
        virtual void writeStartList() override;
        virtual void writeEndList() override;

        virtual void writeNDArray(const std::vector<int>&, const std::string&, const unsigned char*) override;

        virtual void writePrimitive(const int) override;
        virtual void writePrimitive(const long) override;
        virtual void writePrimitive(const float) override;
        virtual void writePrimitive(const double) override;
        virtual void writePrimitive(const std::string&) override;
        virtual void writePrimitive(const bool) override;

        virtual void writeKey(const std::string&) override;

        datanavigator::NavigatorPtr getResult() const
        {
            return lastRemovedToken->getElement();
        }

    private:
        Path generatePath();

    private:
        bool wroteInitialStartContainer = false;
        NavigatorWriterTokenPtr lastRemovedToken = nullptr;
        std::stack<NavigatorWriterTokenPtr> stack;
    };
}
