/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/navigator/data/Navigator.h>

namespace armarx::aron::datanavigator
{

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class upperType##Navigator; \
    typedef std::shared_ptr<upperType##Navigator> upperType##NavigatorPtr;

    HANDLE_PRIMITIVE_DATA
#undef RUN_ARON_MACRO


#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class upperType##Navigator : \
        virtual public Navigator \
    { \
    public: \
        using PointerType = upperType##NavigatorPtr; \
        \
        /* constructors */ \
        upperType##Navigator(const Path& = Path()); \
        upperType##Navigator(const data::Aron##upperType##Ptr&, const Path& = Path()); \
        upperType##Navigator(const lowerType&, const Path& = Path()); \
        \
        /* operators */ \
        operator lowerType() const; \
        bool operator==(const upperType##Navigator&) const; \
        virtual bool equalsDataNavigator(const NavigatorPtr&) const override; \
        bool equalsDeep(const upperType##NavigatorPtr&) const; \
        \
        /* static methods */ \
        static upperType##NavigatorPtr DynamicCast(const NavigatorPtr& n); \
        static upperType##Navigator DynamicCast(Navigator& n); \
        static upperType##NavigatorPtr DynamicCastAndCheck(const NavigatorPtr& n); \
        \
        static upperType##NavigatorPtr FromAron##upperType##Ptr(const data::Aron##upperType##Ptr& aron); \
        static data::Aron##upperType##Ptr ToAron##upperType##Ptr(const upperType##NavigatorPtr& navigator); \
        \
        /* public member functions */ \
        void setValue(const lowerType& x); \
        lowerType getValue() const; \
        \
        data::Aron##upperType##Ptr toAron##upperType##Ptr() const; \
        \
        /* virtual implementations */ \
        virtual data::AronDataPtr getResult() const override; \
        virtual std::string getName() const override; \
        virtual std::vector<NavigatorPtr> getChildren() const override; \
        virtual size_t childrenSize() const override; \
        \
        virtual typenavigator::NavigatorPtr recalculateType() const override; \
        virtual bool fullfillsType(const typenavigator::NavigatorPtr&) const override; \
        \
    private: \
        /* members */ \
        data::Aron##upperType##Ptr aron; \
    };

    HANDLE_PRIMITIVE_DATA
#undef RUN_ARON_MACRO
}
