/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>

namespace armarx::aron::typenavigator
{
    class EigenMatrixNavigator;
    typedef std::shared_ptr<EigenMatrixNavigator> EigenMatrixNavigatorPtr;

    class EigenMatrixNavigator :
        virtual public Navigator
    {
    public:
        using PointerType = EigenMatrixNavigatorPtr;

    public:
        // constructors
        EigenMatrixNavigator() = delete;
        EigenMatrixNavigator(const Path& path);
        EigenMatrixNavigator(const type::AronEigenMatrixPtr&, const Path& path);

        bool checkDimensions(const std::vector<int>&) const;
        bool checkTypename(const std::string&) const;

        unsigned int getRows() const;
        unsigned int getCols() const;
        std::string getTypename() const;
        std::vector<int> getDimensions() const;

        void setRows(const unsigned int&);
        void setCols(const unsigned int&);
        void setTypename(const std::string&);
        void setDimensions(const std::vector<int>&);

        type::AronEigenMatrixPtr toAronEigenMatrixPtr() const;

        // static methods
        static EigenMatrixNavigatorPtr DynamicCast(const NavigatorPtr& n);
        static EigenMatrixNavigatorPtr DynamicCastAndCheck(const NavigatorPtr& n);

        // virtual implementations
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;
        virtual type::AronTypePtr getResult() const override;
        virtual std::string getName() const override;

    public:
        const std::map<std::string, std::vector<std::string>> ACCEPTED_TYPES =
        {
            {"short", {}},
            {"int", {}},
            {"long", {}},
            {"float", {}},
            {"double", {}},
            //{"complex float", {"std::complex<float>"}},
            //{"complex double", {"std::complex<double>"}},
        };
        const unsigned int ACCEPTED_DIMENSION_MIN_SIZE = 2;

    private:
        // members
        type::AronEigenMatrixPtr type;
    };
}
