/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "IVTCByteImage.h"

namespace armarx::aron::typenavigator
{
    // constructors
    IVTCByteImageNavigator::IVTCByteImageNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eIVTCByteImage, path),
        Navigator(type::Descriptor::eIVTCByteImage, path),
        type(new type::AronIVTCByteImage(0, 0, ""))
    {
    }

    IVTCByteImageNavigator::IVTCByteImageNavigator(const type::AronIVTCByteImagePtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eIVTCByteImage, path),
        Navigator(type::Descriptor::eIVTCByteImage, path),
        type(o)
    {
        CheckAronPtrForNull("AronIVTCByteImageTypeNavigator", "AronIVTCByteImageTypeNavigator", getPath(), o);
    }

    std::string IVTCByteImageNavigator::checkTypename(const std::string& s) const
    {
        for (const auto& [key, list] : ACCEPTED_TYPES)
        {
            if (s == key)
            {
                return key;
            }

            for (const auto& el : list)
            {
                if (s == el)
                {
                    return key;
                }
            }
        }
        throw error::AronException("AronIVTCByteImageTypeNavigator", "checkTypename", "The typename is empty.", getPath());
    }

    type::AronIVTCByteImagePtr IVTCByteImageNavigator::toAronIVTCByteImagePtr() const
    {
        checkTypename(type->typeName);
        return type;
    }

    std::vector<int> IVTCByteImageNavigator::getDimensions() const
    {
        return {type->width, type->height};
    }

    unsigned int IVTCByteImageNavigator::getWidth() const
    {
        return type->width;
    }

    unsigned int IVTCByteImageNavigator::getHeight() const
    {
        return type->height;
    }

    std::string IVTCByteImageNavigator::getTypename() const
    {
        return type->typeName;
    }

    void IVTCByteImageNavigator::setWidth(const unsigned int& w)
    {
        if (w == 0)
        {
            throw error::AronException("AronIVTCByteImageTypeNavigator", "setWidth", "The width of an image cannot be 0", getPath());
        }
        type->width = w;
    }

    void IVTCByteImageNavigator::setHeight(const unsigned int& h)
    {
        if (h == 0)
        {
            throw error::AronException("AronIVTCByteImageTypeNavigator", "setHeight", "The height of an image cannot be 0", getPath());
        }
        type->height = h;
    }

    void IVTCByteImageNavigator::setTypename(const std::string& u)
    {
        type->typeName = checkTypename(u);
    }

    // static methods
    IVTCByteImageNavigatorPtr IVTCByteImageNavigator::DynamicCast(const NavigatorPtr& n)
    {
        return std::dynamic_pointer_cast<IVTCByteImageNavigator>(n);
    }

    IVTCByteImageNavigatorPtr IVTCByteImageNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckTypeNavigatorPtrForNull("AronIVTCByteImageTypeNavigator", "DynamicCast[Before]", n);
        IVTCByteImageNavigatorPtr casted = IVTCByteImageNavigator::DynamicCast(n);
        CheckTypeNavigatorPtrForNull("AronIVTCByteImageTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
        return casted;
    }

    // virtual implementations
    std::vector<NavigatorPtr> IVTCByteImageNavigator::getChildren() const
    {
        return {};
    }

    size_t IVTCByteImageNavigator::childrenSize() const
    {
        return 0;
    }

    type::AronTypePtr IVTCByteImageNavigator::getResult() const
    {
        return toAronIVTCByteImagePtr();
    }

    std::string IVTCByteImageNavigator::getName() const
    {
        return "AronIVTCByteImageType<" + simox::alg::to_string(getDimensions(), ", ") + ", " + type->typeName + ">";
    }
}

