/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Position.h"

namespace armarx::aron::typenavigator
{
    // constructors
    PositionNavigator::PositionNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::ePosition, path),
        Navigator(type::Descriptor::ePosition, path),
        type(new type::AronPosition())
    {
    }

    PositionNavigator::PositionNavigator(const type::AronPositionPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::ePosition, path),
        Navigator(type::Descriptor::ePosition, path),
        type(o)
    {
        CheckAronPtrForNull("AronPositionTypeNavigator", "AronPositionTypeNavigator", getPath(), o);
    }

    type::AronPositionPtr PositionNavigator::toAronPositionPtr() const
    {
        return type;
    }

    std::vector<int> PositionNavigator::getDimensions() const
    {
        return ACCEPTED_DIMENSION;
    }

    std::string PositionNavigator::getTypename() const
    {
        return ACCEPTED_TYPE;
    }

    // static methods
    PositionNavigatorPtr PositionNavigator::DynamicCast(const NavigatorPtr& n)
    {
        return std::dynamic_pointer_cast<PositionNavigator>(n);
    }

    PositionNavigatorPtr PositionNavigator::DynamicCastAndCheck(const NavigatorPtr& n)
    {
        CheckTypeNavigatorPtrForNull("AronPositionTypeNavigator", "DynamicCast[Before]", n);
        PositionNavigatorPtr casted = PositionNavigator::DynamicCast(n);
        CheckTypeNavigatorPtrForNull("AronPositionTypeNavigator", "DynamicCast[After]", n->getPath(), casted);
        return casted;
    }

    // virtual implementations
    std::vector<NavigatorPtr> PositionNavigator::getChildren() const
    {
        return {};
    }

    size_t PositionNavigator::childrenSize() const
    {
        return 0;
    }

    type::AronTypePtr PositionNavigator::getResult() const
    {
        return toAronPositionPtr();
    }

    std::string PositionNavigator::getName() const
    {
        return "AronPositionType";
    }
}

